;Quite often you need to change a group of objects to a Reference Layer.
;This allows you to distinguish between reference objects and new objects
;and also allows you to plot the reference objects using a lighter pen.
;Unfortunately, when you change these objects to the reference layer,
;their colour changes to the Reference Layers default linetype as well
;as to the Reference Layers default colour.
;This routine changes all objects to the Reference Layer but retains
;their original linetype. You can also select a different colour from that
;of the layers default colour if you so wish.
;The routine also allows you to filter out selected layers that will not
;be changed to the Reference layer.

;**************************************************************************
;*             Written by Kenny Ramage - January 1999                     *
;*                E-Mail : kramage@namdeb.com.na                          *
;*           URL : http://members.tripod.com/~kramage/                    *
;**************************************************************************

;Usage :

;To load type (load "refl").
;Type "refl" after loading.
;A dialogue box will appear with two list boxes and a colour selection button.
;The left hand dialogue box contains a list of all layers currently defined
;in the drawing. Choose one of these layers to use as your Reference Layer.
;The default colour of the current choosen layer will appear in the colour
;image box.
;The left hand dialogue box also contains a list of all the current layers.
;Choosing one or more of these layers will remove any objects on these layers
;from the selection set to be changed.
;(To choose more than one layer to omitt, hold down the Ctrl key.)
;You can also change the default colour by clicking on the Colour Button.
;This will bring up a colour dialogue box. The new colour choosen will appear
;in the colour image box.

;**************************************************************************

(prompt "\nChange to Reference Layer written by Kenny Ramage.
        \nType Refl to start.......")

(defun c:refl ( / names siz1 siz userclick c2 f t1 b dcl_id w h c3 siz3
		  siz2 a h index nu b1 d e)
	(initerr)
	(setvar "cmdecho" 0)

 	(setq siz1 "0")
	(setq siz " ")
	(setq userclick T)

	(setq c2 (cdr (assoc 62
        (tblsearch "Layer" "0"))))
	(setq f 1)
	(while
	(setq t1 (tblnext "Layer" f))
	(setq f nil)
	(setq b (cdr (assoc 2 t1)))
	(if (/= b "DEFPOINTS")
	(setq NAMES (append NAMES (list b)))
	);if
	);while


  (setq dcl_id (load_dialog "refl.dcl"))

  (if (not (new_dialog "refl" dcl_id)

      );not

    (exit)

  );if
  
  (set_tile "sel1" "0")
  (set_tile "txt1" (itoa c2))

  (setq w (dimx_tile "im")
        h (dimy_tile "im")

);setq


(defun ccc ()
  (start_image "im")
  (fill_image 0 0 w h c2)
  (end_image)	
)

  (ccc)

  (action_tile "col"
   "(setq c2 (acad_colordlg c2 nil))
    (setq c3 c2)
    (set_tile \"txt1\" (itoa c2))
    (ccc)"
  );action_tile

  (action_tile "sel1"
   "(setq SIZ3 (get_tile \"sel1\"))
    (setq SIZ3 (nth (atoi SIZ3) NAMES))
    (setq c2 (cdr (assoc 62 (tblsearch \"Layer\" SIZ3))))
    (set_tile \"txt1\" (itoa c2))
    (ccc)"
  );action_tile

  (start_list "select2")
  (mapcar 'add_list NAMES)
  (end_list)

  (start_list "sel1")
  (mapcar 'add_list NAMES)
  (end_list)
    (action_tile
    "cancel"	
    "(done_dialog) (setq userclick nil)"
    );action_tile

  (action_tile
    "accept"	
    (strcat	
      "(progn 
       (setq SIZ (get_tile \"select2\"))"
       "(setq SIZ1 (get_tile \"sel1\"))"	
       " (done_dialog)(setq userclick T))"	
    );strcat
  );action tile

  (start_dialog)	

  (unload_dialog dcl_id)	

   (if userclick	
    (progn
      (if siz
      (setq SIZ (mklist siz))
      );if	
      (setq SIZ2 (nth (atoi SIZ1) NAMES))
      (if (not c3)
      (setq c2 (cdr (assoc 62 (tblsearch "Layer" SIZ2))))
      )
	(setq a (ssget))
	(setq n (sslength a))
	(setq index 0)
	(setq nu 0)
	(repeat n
	   (princ (strcat "\rFiltering and Processing selected objects..."
               (setq spnbar (spinbar spnbar))))
	  (setq b (ssname a index))
	  (setq b1 (entget b))
	  (setq index (1+ index))
	  (setq c (cdr (assoc 8 b1)))
          (setq d (tblsearch "LAYER" c))
	  (setq e (cdr (assoc 6 d)))
	  (if (not (member c SIZ))
          (progn
	  (setq nu (1+ nu))
	  (command "Change" b "" "p" "LA" SIZ2 "")
	  (command "Change" b "" "p" "C" c2 "")
	  (command "Change" b "" "p" "LT" e "")
	  );progn
	 );if
	);repeat
	(princ "\n")
	(princ nu)
	(princ " Objects Changed to Layer: ")
	(princ siz2)
	(princ "  Colour: ")
	(princ c2)
	
    );progn

  );if userclick
  (reset)

(princ)

);defun C:Refl

(defun mklist (readlist / count item retlist)

	(setq count 1)
	(while (setq item (read readlist))
		(setq retlist (cons (nth item names) retlist))
		(while (and (/= " " (substr readlist count 1))
		   (/= "" (substr readlist count 1)))
		   (setq count (1+ count))
		)
		(setq readlist (substr readlist count))
	)
	(reverse retlist)
)

;============================================================
;;;BEGIN SPIN BAR
(defun spinbar (spnbar)
     (cond
          ((= spnbar "\\") "|")
          ((= spnbar "|") "/")
          ((= spnbar "/") "-")
          (t "\\")
     );cond
);spinbar
;=============================================================
(defun initerr ()
  (setq oldlayer (getvar "clayer"))
  (setq oldsnap (getvar "osmode"))
  (setq oldecho (getvar "cmdecho"))
  (setq temperr *error*)
  (setq *error* trap)
  (princ)
)		
;;;*===========================================================
(defun trap (errmsg)
  (command nil nil nil)
  (if (not (member errmsg '("console break" "Function Cancelled"))
      ) ;_ end of not
    (princ (strcat "\nError: " errmsg))
  )					;                 
  (setvar "osmode" oldsnap)
  (setvar "cmdecho" oldecho)
  (princ "\nError Resetting Enviroment ")
  (terpri)
  (setq *error* temperr)
  (princ)
)					;defun
;;;*===========================================================
(defun reset ()
  (setq *error* temperr)
  (setvar "osmode" oldsnap)
  (setvar "cmdecho" oldecho)
  (princ)
)					;defun
;;;*======================================================		

(princ)
